@@ -113,86 +113,69 @@ def _diff2msg(diff, tol=None, dt_as_gpsweek=False):
     return msg
 
 
-def _compare_sv_states(diffstd, log_lvl, tol=None, plot=False):
-    diff_sv = diffstd[diffstd.attrs["SAT_MASK"]].droplevel("NUM", axis=0).unstack(["TYPE", "SITE", "SAT", "BLK"])
-    if diff_sv.empty:
-        _logging.warning(msg=f":diffutil SVs states not present. Skipping")
+def _compare_states(diffstd: _pd.DataFrame, log_lvl: int, tol: _Union[float, None] = None, plot: bool = False) -> int:
+    """_summary_
+
+    Args:
+        diffstd (_pd.DataFrame): a difference DataFrame to assess
+        log_lvl (int): logging level of the produced messages
+        tol (_Union[float, None], optional): Either a float threshold or None to use the present STD values. Defaults to None.
+        plot (bool, optional): So you want a simple plot to terminal? Defaults to False.
+
+    Returns:
+        int: status (0 means differences within threshold)
+    """
+    diff_states = diffstd.unstack(["TYPE", "SITE", "SAT", "BLK", "NUM"])
+    # we remove the '.droplevel("NUM", axis=0)' due to ORBIT_PTS non-uniqueness. Changing to ORBIT_PTS_blah might be a better solution
+    if diff_states.empty:
+        _logging.warning(msg=f":diffutil states not present. Skipping")
         return 0
     if plot:
         # a standard scatter plot
-        _gn_plot.diff2plot(diff_sv.DIFF.PHASE_BIAS, kind=None, title="PHASE BIAS DIFF")
+        _gn_plot.diff2plot(diff_states.DIFF.PHASE_BIAS, kind=None, title="PHASE BIAS DIFF")
 
         # a bar plot of mean values
-        diffstd_mean = diff_sv.DIFF.PHASE_BIAS.mean(axis=0)
+        diffstd_mean = diff_states.DIFF.PHASE_BIAS.mean(axis=0)
         diffstd_mean.index = diffstd_mean.index.to_series().astype(str)
         _gn_plot.diff2plot(diffstd_mean, kind="bar", title="MEAN PHASE BIAS DIFF")
-    bad_sv_states = _diff2msg(diff_sv, tol)
-    if bad_sv_states is not None:
+    bad_states = _diff2msg(diff_states, tol)
+    if bad_states is not None:
         _logging.log(
-            msg=f':diffutil found SV states diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_sv_states.to_string(justify="center")}\n',
+            msg=f':diffutil found states diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_states.to_string(justify="center")}\n',
             level=log_lvl,
         )
         return -1
     _logging.log(
-        msg=f':diffutil [OK] SVs states diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
+        msg=f':diffutil [OK] states diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
         level=_logging.INFO,
     )
     return 0
 
 
-def _compare_nonsv_states(diffstd, log_lvl, tol=None):
-    diff_nonsv = diffstd[~diffstd.attrs["SAT_MASK"]].droplevel("SAT", axis=0).unstack(["TYPE", "SITE", "NUM", "BLK"])
-    if diff_nonsv.empty:
-        _logging.warning(msg=f":diffutil non-SVs states not present. Skipping")
+def _compare_residuals(diffstd: _pd.DataFrame, log_lvl: int, tol: _Union[float, None] = None):
+    """Compares extracted POSTFIT residuals from the trace file and generates a comprehensive statistics on the present differences. Alternatively logs an OK message.
+
+    Args:
+        diffstd (_pd.DataFrame): a difference DataFrame to assess
+        log_lvl (int): logging level of the produced messages
+        tol (_Union[float, None], optional): Either a float threshold or None to use the present STD values. Defaults to None.
+
+    Returns:
+        int: status (0 means differences within threshold)
+    """
+    diff_residuals = diffstd.unstack(["TYPE", "NUM", "SAT", "SITE", "BLK"])
+    if diff_residuals.empty:
+        _logging.warning(f":diffutil residuals not present. Skipping")
         return 0
-    bad_nonsv_states = _diff2msg(diff_nonsv, tol=tol)
-    if bad_nonsv_states is not None:
+    bad_residuals = _diff2msg(diff_residuals, tol=tol)
+    if bad_residuals is not None:
         _logging.log(
-            msg=f':diffutil non-SVs states diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_nonsv_states.to_string(justify="center")}\n',
+            msg=f':diffutil found residuals diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_residuals.to_string(justify="center")}\n',
             level=log_lvl,
         )
         return -1
     _logging.log(
-        msg=f':diffutil [OK] non-SVs states diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
-        level=_logging.INFO,
-    )
-    return 0
-
-
-def _compare_sat_postfit_residuals(diffstd, log_lvl, tol=None):
-    diff_count = diffstd[diffstd.attrs["SAT_MASK"]].unstack(["TYPE", "NUM", "SAT", "SITE", "BLK"])
-    if diff_count.empty:
-        _logging.warning(f":diffutil SVs residuals not present. Skipping")
-        return 0
-    bad_sv_residuals = _diff2msg(diff_count, tol=tol)
-    if bad_sv_residuals is not None:
-        _logging.log(
-            msg=f':diffutil found SVs residuals diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_sv_residuals.to_string(justify="center")}\n',
-            level=log_lvl,
-        )
-        return -1
-    _logging.log(
-        msg=f':diffutil [OK] SVs residuals diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
-        level=_logging.INFO,
-    )
-    return 0
-
-
-def _compare_nonsat_postfit_residuals(diffstd, log_lvl, tol=None):
-    diff_count = diffstd[~diffstd.attrs["SAT_MASK"]]
-    if diff_count.empty:  # no non-sat records found such as Zamb
-        _logging.warning(f":diffutil non-SVs residuals not present. Skipping")
-        return 0
-    diff_count = diff_count.droplevel("SAT").unstack(["TYPE", "NUM", "SITE", "BLK"])
-    bad_nonsv_residuals = _diff2msg(diff_count, tol=tol)
-    if bad_nonsv_residuals is not None:
-        _logging.log(
-            msg=f':diffutil found non-SVs residuals diffs above {"the extracted STDs" if tol is None else f"{tol:.1E} tolerance"}:\n{bad_nonsv_residuals.to_string(justify="center")}\n',
-            level=log_lvl,
-        )
-        return -1
-    _logging.log(
-        msg=f':diffutil [OK] non-SVs residuals diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
+        msg=f':diffutil [OK] residuals diffs within the {"extracted STDs" if tol is None else f"{tol:.1E} tolerance"}',
         level=_logging.INFO,
     )
     return 0
@@ -218,7 +201,8 @@ def _compare_stec(diffstd, log_lvl, tol=None):
 
 
 def difftrace(trace1_path, trace2_path, tol, std_coeff, log_lvl, plot):
-    """Compares two trace/sum files"""
+    """Compares two trace/sum files
+    Only POSTFIT residuals are compared"""
     trace1 = _gn_io.common.path2bytes(trace1_path)
     trace2 = _gn_io.common.path2bytes(trace2_path)
 
@@ -237,8 +221,7 @@ def difftrace(trace1_path, trace2_path, tol, std_coeff, log_lvl, plot):
         diffstd_states = _valvar2diffstd(states1.iloc[:, :2], states2.iloc[:, :2], std_coeff=std_coeff, trace=True)
         if diffstd_states.attrs["EXTRA_SATS"] != []:
             _logging.warning(msg=f':diffutil found no counterpart for: {sorted(diffstd_states.attrs["EXTRA_SATS"])}')
-        status += _compare_sv_states(diffstd_states, tol=tol, log_lvl=log_lvl, plot=plot)
-        status += _compare_nonsv_states(diffstd_states, tol=tol, log_lvl=log_lvl)
+        status += _compare_states(diffstd_states, tol=tol, log_lvl=log_lvl, plot=plot)
 
     _logging.info(msg=f":diffutil -----testing trace residuals-----")
     resids1 = _gn_io.trace._read_trace_residuals(trace1)
@@ -251,17 +234,18 @@ def difftrace(trace1_path, trace2_path, tol, std_coeff, log_lvl, plot):
     else:
         _gn_aux.throw_if_index_duplicates(resids1)
         _gn_aux.throw_if_index_duplicates(resids2)
-        diffstd_residuals = _valvar2diffstd(resids1.iloc[:, 2:], resids2.iloc[:, 2:], std_coeff=std_coeff, trace=True)
-        status += _compare_sat_postfit_residuals(diffstd_residuals, tol=tol, log_lvl=log_lvl)
-        status += _compare_nonsat_postfit_residuals(diffstd_residuals, tol=tol, log_lvl=log_lvl)
+        diffstd_residuals = _valvar2diffstd(
+            resids1[["POSTFIT", "STD"]], resids2[["POSTFIT", "STD"]], std_coeff=std_coeff, trace=True
+        )
+        status += _compare_residuals(diffstd_residuals, tol=tol, log_lvl=log_lvl)
 
     return status
 
 
 def diffsnx(snx1_path, snx2_path, tol, std_coeff, log_lvl):
     """Compares two sinex files"""
-    snx1_df = _gn_io.sinex._get_snx_vector(path_or_bytes=snx1_path, stypes=("EST",), format='long', verbose=True)
-    snx2_df = _gn_io.sinex._get_snx_vector(path_or_bytes=snx2_path, stypes=("EST",), format='long', verbose=True)
+    snx1_df = _gn_io.sinex._get_snx_vector(path_or_bytes=snx1_path, stypes=("EST",), format="long", verbose=True)
+    snx2_df = _gn_io.sinex._get_snx_vector(path_or_bytes=snx2_path, stypes=("EST",), format="long", verbose=True)
 
     if (snx1_df is None) or (snx2_df is None):
         return -1  # don't wait, throw error right away as smth bad happened, errors have been logged
