@@ -5,28 +5,24 @@ erp
 clk
 rnx (including transformation from crx to rnx)
 """
+import datetime as _datetime
+import logging
+import shutil
+import subprocess as _subprocess
+import sys as _sys
+import threading
+import time as _time
 from ftplib import FTP_TLS as _FTP_TLS
 from pathlib import Path as _Path
-import urllib.request as _request
-import subprocess as _subprocess
-import datetime as _datetime
-import urllib.error as _err
-import random as _random
-import time as _time
-import pandas as _pd
-import numpy as _np
-import sys as _sys
-import logging
-import os as _os
-import shutil
+from urllib import request as _request
+from urllib.error import HTTPError as _HTTPError
 
-from .gn_datetime import gpsweekD, dt2gpswk, gpswkD2dt
-
-
-import threading
 import boto3
+import numpy as _np
+import pandas as _pd
 from boto3.s3.transfer import TransferConfig
 
+from .gn_datetime import dt2gpswk, gpsweekD, gpswkD2dt
 
 MB = 1024 * 1024
 
@@ -107,27 +103,33 @@ def upload_with_chunksize_and_meta(
     # return transfer_callback.thread_info
 
 
-def request_checksum(url):
+def request_metadata(url: str, max_retries: int = 5, metadata_header: str = "x-amz-meta-md5checksum") -> str:
     """requests md5checksum metadata over https (AWS S3 bucket)
-    Returns None if file not found (404) or if no md5checksum metadata exists"""
+    Returns None if file not found (404) or if md5checksum metadata key does not exist"""
     logging.info(f'requesting checksum for "{url}"')
-    md5_checksum_response = None
-    try:
-        with _request.urlopen(url) as response:
-            if response.status == 200:
-                logging.info(msg="server says OK")
-                md5_checksum_response = response.getheader("x-amz-meta-md5checksum")
-                logging.info(f'Got "{md5_checksum_response}"')
-    except _err.HTTPError as e:
-        logging.error(e)  # TODO add a more explicit message
-    return md5_checksum_response
+    for retry in range(1, max_retries + 1):
+        try:
+            with _request.urlopen(url) as response:
+                if response.status == 200:
+                    logging.info(msg="server says OK")
+                    url_metadata = response.getheader(metadata_header)
+                    logging.info(f'Got "{url_metadata}"')
+                    return url_metadata  # md5_checksum is None if no md5 metadata present
+        except _HTTPError as err:
+            logging.error(f" HTTP Error {err.code} for {url}: {err.reason}: Returning 'None' as checksum")
+            if err.code == 404:
+                return None  # File Not Found on the server so no checksum exists
+            t_seconds = 2**retry
+            logging.error(f"Retry No. {retry} in {t_seconds} seconds")
+            _time.sleep(t_seconds)
+            if retry >= max_retries:
+                logging.error(f"Maximum number of retries reached: {max_retries}")
+                raise (err)
 
 
-def download_url(url, destfile, max_retries=5):
+def download_url(url: str, destfile: str, max_retries: int = 5):
     logging.info(f'requesting "{url}"')
-    retries = 0
-    download_done = False
-    while not download_done and retries <= max_retries:
+    for retry in range(1, max_retries + 1):
         try:
             with _request.urlopen(url) as response:
                 if response.status == 200:
@@ -135,25 +137,30 @@ def download_url(url, destfile, max_retries=5):
                     with open(destfile, "wb") as out_file:
                         shutil.copyfileobj(response, out_file)
             return 0
-        except _err.HTTPError as e:
-            logging.error(e)  # TODO add a more explicit message
-            _time.sleep(_random.uniform(0.0, 2.0 ** retries))
-            retries += 1
-    logging.info(f"Maximum number of retries reached: {max_retries}. File not downloaded")
-    return None
+        except _HTTPError as err:
+            logging.error(f" HTTP Error {err.code}: {err.reason}")
+            if err.code == 404:
+                return None  # File Not Found on the server so no point in retrying
+            t_seconds = 2**retry
+            logging.error(f"Retry No. {retry} in {t_seconds} seconds")
+            _time.sleep(t_seconds)
+            if retry >= max_retries:
+                logging.error(f"Maximum number of retries reached: {max_retries}. File not downloaded")
+                return None
 
 
 def gen_uncomp_filename(comp_filename):
-    """Name of Uncompressed filename given the compressed name"""
+    """Name of uncompressed filename given the compressed name"""
     if comp_filename.endswith(".crx.gz"):
-        uncomp_file = comp_filename[:-6] + "rnx"
+        return comp_filename[:-6] + "rnx"
     elif comp_filename.endswith(".gz"):
-        uncomp_file = comp_filename[:-3]
+        return comp_filename[:-3]
     elif comp_filename.endswith(".Z"):
-        uncomp_file = comp_filename[:-2]
+        return comp_filename[:-2]
+    elif comp_filename.endswith(".bz2"):
+        return comp_filename[:-4]
     else:
-        uncomp_file = comp_filename
-    return uncomp_file
+        return comp_filename
 
 
 def gen_prod_filename(dt, pref, suff, f_type, wkly_file=False, repro3=False):
@@ -756,12 +763,12 @@ def download_rinex3(dates, stations, dest, dwn_src="cddis", ftps=False, f_dict=F
 
 
 def get_vars_from_file(path):
-    from importlib.util import spec_from_loader, module_from_spec
     from importlib.machinery import SourceFileLoader
+    from importlib.util import module_from_spec, spec_from_loader
 
     spec = spec_from_loader("tags", SourceFileLoader("tags", path))
     tags = module_from_spec(spec)
     spec.loader.exec_module(tags)
 
-    tags_dict = ({item:getattr(tags, item) for item in dir(tags) if not item.startswith("__")})
+    tags_dict = {item: getattr(tags, item) for item in dir(tags) if not item.startswith("__")}
     return tags_dict
