@@ -7,6 +7,7 @@ from . import (
     discon,
     igslog,
     ionex,
+    nanu,
     pea,
     pod,
     psd,
@@ -27,6 +28,7 @@ __all__ = [
     "discon",
     "igslog",
     "ionex",
+    "nanu",
     "pea",
     "pod",
     "psd",
