@@ -18,10 +18,7 @@ def read_pea_partials(path):
         names=[None, "MJD", "TOD", "X", "Y", "Z"],
     )
     df_done = df[["X", "Y", "Z"]].set_index(
-        [
-            _gn_datetime.mjd2j2000(df.MJD.values, df.TOD.values, pea_partials=True),
-            df.iloc[:, 0].astype(_gn_const.PRN_CATEGORY),
-        ]
+        [_gn_datetime.mjd2j2000(df.MJD.values, df.TOD.values, pea_partials=True), df.iloc[:, 0]]
     )
     df_partials = _pd.concat([df_done], keys=["EST"], axis=1) / 1000
     df_partials.attrs["path"] = path
