@@ -4,20 +4,21 @@ import logging as _logging
 import re as _re
 import zlib as _zlib
 from io import BytesIO as _BytesIO
+from typing import Union as _Union
 
 import numpy as _np
 import pandas as _pd
 
+from .. import gn_aux as _gn_aux
 from .. import gn_const as _gn_const
 from .. import gn_datetime as _gn_datetime
-from .. import gn_aux as _gn_aux
 from .. import gn_io as _gn_io
 
 _RE_BLK_HEAD = _re.compile(rb"\+\w+\/\w+(\s[LU]|)\s*(CORR|COVA|INFO|)[ ]*\n(?:\*[ ].+\n|)(?:\*\w.+\n|)")
 _RE_STATISTICS = _re.compile(r"^[ ]([A-Z (-]+[A-Z)])[ ]+([\d+\.]+)", _re.MULTILINE)
 
 
-def get_snx_header(path_or_bytes):
+def _get_snx_header(path_or_bytes):
     # read line that starts with %=SNX and parse it into dict
     snx_bytes = _gn_io.common.path2bytes(path_or_bytes)
     header_begin = snx_bytes.find(b"%=SNX")
@@ -44,7 +45,7 @@ def format_snx_header(h: dict):
     ).tolist()
     # we override the format version with 2.02
     line = f"%=SNX 2.02 {h['ac_created']} {dates_yydoysec[0]} {h['ac_data_prov']} {dates_yydoysec[1]} {dates_yydoysec[2]} {h['obs_code']} {h['n_estimates']:05} {h['constr_code']:1} {' '.join(h['content'])}"
-    return line
+    return [line]
 
 
 def snx_soln_str_to_int(soln: _pd.Series) -> _pd.Series:
@@ -127,11 +128,12 @@ def _snx_extract(snx_bytes, stypes, obj_type, verbose=True):
     objects_in_buf = 0
     for stype in stypes:
         if stype in stypes_dict.keys():
-            remove_header = objects_in_buf != 0
-            if (objects_in_buf == 0) & (obj_type == "MATRIX"):  # override matrix header as comments may be present
-                snx_buffer += b"*PARA1 PARA2 ____PARA2+0__________ ____PARA2+1__________ ____PARA2+2__________\n"
-                remove_header = True
-            stype_extr = _snx_extract_blk(snx_bytes=snx_bytes, blk_name=stypes_dict[stype], remove_header=remove_header)
+            if objects_in_buf == 0:  # override matrix header as comments may be present
+                if obj_type == "MATRIX":
+                    snx_buffer += b"*PARA1 PARA2 ____PARA2+0__________ ____PARA2+1__________ ____PARA2+2__________\n"
+                elif obj_type == "VECTOR":
+                    snx_buffer += b"*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __ESTIMATED_VALUE____ _STD_DEV___\n"
+            stype_extr = _snx_extract_blk(snx_bytes=snx_bytes, blk_name=stypes_dict[stype], remove_header=True)
             if stype_extr is not None:
                 snx_buffer += stype_extr[0]
                 stypes_rows[stype] = stype_extr[1]
@@ -211,47 +213,69 @@ def _get_snx_matrix(path_or_bytes, stypes=("APR", "EST"), verbose=True):
     return output, stypes_content
 
 
-def snxdf2xyzdf(snxdf, unstack=True, keep_all_soln=None):
-    types_mask = snxdf.TYPE.isin(
-        [
-            "STAX",
-            "STAY",
-            "STAZ",
-            "VELX",
-            "VELY",
-            "VELZ",
-        ]
-    ).values
-    snxdf.drop(index=snxdf.index.values[~types_mask], inplace=True)
-    snx_df = snxdf.set_index(["CODE_PT", "REF_EPOCH", "TYPE"])
+def snxdf2xyzdf(snxdf: _pd.DataFrame, unstack: bool = True, keep_all_soln: _Union[bool, None] = None) -> _pd.DataFrame:
+    """Provides simple functionality to preprocess different variations of the
+    sinex vector dataframe for further processing and analysis.
+
+    Args:
+        snxdf (_pd.DataFrame): 'raw' vector dataframe (see _get_snx_vector's format)
+        unstack (bool, optional): whether to unstack TYPE to columns (STAX, STAY and STAZ) or keep as in sinex file. Defaults to True.
+        keep_all_soln (_Union, optional): drops all the extra solutions if False, leaving just last one (max SOLN for each parameter). If None - keeps all solutions but drops the SOLN index TODO potentially remove the None option. Defaults to None.
+
+    Returns:
+        _pd.DataFrame: a formatted sinex dataframe filtered by STAX, STAY and STAZ types, optionally unstacked by TYPE
+    """
+    snxdf = snxdf.loc(axis=0)[
+        ["STAX", "STAY", "STAZ", "VELX", "VELY", "VELZ"],
+    ]
+
     if keep_all_soln is None:
-        # Retain existing behaviour by (all solutions but no solution number) default. Specifying level stops performance warning
-        snx_df = snx_df.drop(columns=["SOLN"], level=0)
+        # Retain existing behaviour by (all solutions but no solution number) default
+        snx_df = snxdf.reset_index("SOLN", drop=True)
     elif keep_all_soln:
         # Keep all solutions but include SOLN as an index value, not a column to avoid reshape issues
-        snx_df = snx_df.set_index("SOLN", append=True)
+        snx_df = snxdf
     else:
         # Keep just 1 solution by sorting the high solutions to the toping, grouping things by all indices
         # so we get a group for each site, time, pos/vel triplet, and then take the first element of that
         # group, which will be the highest numbered solution thanks to the previous sort
-        snx_df = snx_df.sort_values("SOLN", ascending=False).groupby(level=snx_df.index.names).head(1)
-        snx_df = snx_df.drop(columns=["SOLN"])
-    return snx_df.unstack(2) if unstack else snx_df
+        snx_df = (
+            snx_df.sort_index(axis="index", level="SOLN", ascending=False).groupby(level=["TYPE", "CODE_PT"]).head(1)
+        )
+        snx_df = snx_df.reset_index("SOLN", drop=True)  # TODO do we need to drop SOLN?
+
+    if unstack:
+        snx_df = snx_df.unstack(0)
+        snx_df.attrs = snxdf.attrs  # unstacking drops all attrs so we copy over
+    return snx_df
 
 
 def _get_snx_vector(
-    path_or_bytes,
-    stypes=("EST", "APR"),
-    format="long",
-    keep_all_soln=None,
-    verbose=True,
-    recenter_epochs=False,
-):
-    """stypes = "APR","EST","NEQ"
-    APRIORY, ESTIMATE, NORMAL_EQUATION
+    path_or_bytes: _Union[str, bytes],
+    stypes: tuple = ("EST", "APR"),
+    format: str = "long",
+    keep_all_soln: _Union[bool, None] = None,
+    verbose: bool = True,
+    recenter_epochs: bool = False,
+) -> _pd.DataFrame:
+    """Main function of reading vector data from sinex file. Doesn't support sinex files from EMR AC as APRIORI and ESTIMATE indices are not in sync (APRIORI params might not even exist in he ESTIMATE block). While will parse the file, the alignement of EST and APR values might be wrong. No easy solution was found for the issue thus unsupported for now. TODO parse header and add a warning if EMR agency
 
-    recenter_epochs would override the read-in time values with _gn_const.SEC_IN_DAY // 2 so same-day values from different sinex files will align. The actual timestamps could be close to 43200 but not exactly. This switch insures the alignment
+    Args:
+        path_or_bytes (_Union): _description_
+        stypes (tuple, optional): Specifies which blocks to extract: APRIORY, ESTIMATE, NORMAL_EQUATION. Could contain any from "APR","EST" and "NEQ". Defaults to ("EST", "APR").
+        format (str, optional): format of the output dataframe: one of 'raw', 'wide' and 'long. Defaults to "long". TODO. shall the keys be allcaps and how about creating a pandas subclass (bad idea?) or similar
+        keep_all_soln (_Union, optional): whether to keep all solutions of each parameter or just keep the one with max SOLN. If None then keeps all but drops SOLN index. Defaults to None. TODO do we need None option?
+        verbose (bool, optional): logs extra information which might be useful for debugging. Defaults to True.
+        recenter_epochs (bool, optional): overrides the read-in time values with _gn_const.SEC_IN_DAY // 2 so same-day values from different sinex files will align as the actual timestamps could be close to 43200 but not exactly. Defaults to False.
+
+    Raises:
+        ValueError: if float data is bad, usually means that file is corrupted
+        NotImplementedError: for the unknown format option
+
+    Returns:
+        _pd.DataFrame: a dataframe of sine vector block/blocks
     """
+
     path = None
     if isinstance(path_or_bytes, str):
         path = path_or_bytes
@@ -263,30 +287,24 @@ def _get_snx_vector(
         snx_bytes = path_or_bytes
 
     n_header = int(snx_bytes[60:65])
-
-    if stypes == ("NEQ"):
-        stypes = ("APR", "NEQ")
-        # should always return NEQ vector with APR above it
-        if verbose:
-            _logging.info("Prepending APR")
-
     stypes = _get_valid_stypes(stypes)  # EST is always first as APR may have skips
 
     extracted = _snx_extract(snx_bytes=snx_bytes, stypes=stypes, obj_type="VECTOR", verbose=verbose)
     if extracted is None:
         return None
     snx_buffer, stypes_rows, stypes_form, stypes_content = extracted
-    stypes = stypes_rows.keys()
+
+    BLK_TYPE = _np.repeat(list(stypes_rows.keys()), list(stypes_rows.values()))
 
     try:
         vector_raw = _pd.read_csv(
             snx_buffer,
             delim_whitespace=True,
-            comment=b"*",
-            header=None,
-            usecols=[1, 2, 3, 4, 5, 6, 7, 8, 9],
-            names=["TYPE", "CODE", "PT", "SOLN", "REF_EPOCH", "UNIT", "CONSTR", "EST", "STD"],
+            header=0,
+            usecols=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
+            names=["INDEX", "TYPE", "CODE", "PT", "SOLN", "REF_EPOCH", "UNIT", "CONSTR", "VAL", "STD"],
             dtype={
+                "INDEX": int,  # TODO might need to switch to _pd.Int64Dtype() so NaNs on unstack do not make it fallback to float
                 "TYPE": object,
                 "CODE": object,
                 "PT": object,
@@ -294,8 +312,8 @@ def _get_snx_vector(
                 "REF_EPOCH": object,
                 "UNIT": _gn_const.UNIT_CATEGORY,
                 "CONSTR": int,
-                "EST": _np.float_,
-                "STD": _np.float_,
+                "VAL": float,
+                "STD": float,
             },
         )
 
@@ -306,23 +324,16 @@ def _get_snx_vector(
         else:
             raise _e
 
-    stypes_vals = (
-        vector_raw.iloc[:, -2:].values.reshape(len(stypes), -1, 2).T
-    )  # converting the consecutively structured VALs and STDs of stypes blocks into vertically stacked arrays of VAls-STDs pairs for each stype
-    df = vector_raw.iloc[: stypes_vals.shape[1], :5]
-    df.attrs["aux"] = vector_raw.iloc[: stypes_vals.shape[1], [5, 6]]
+    vector_raw["CODE_PT"] = vector_raw.CODE.values + "_" + vector_raw.PT.values
+    vector_raw.drop(["CODE", "PT"], axis="columns", inplace=True)
+    vector_raw.REF_EPOCH = _gn_datetime.yydoysec2datetime(vector_raw.REF_EPOCH, recenter=recenter_epochs, as_j2000=True)
+    vector_raw.SOLN = snx_soln_str_to_int(vector_raw.SOLN)
 
-    df.REF_EPOCH = _gn_datetime.yydoysec2datetime(df.REF_EPOCH, recenter=recenter_epochs, as_j2000=True)
-    df.SOLN = snx_soln_str_to_int(df.SOLN)
-    df["CODE_PT"] = df.CODE.values + "_" + df.PT.values
-    df.drop(["CODE", "PT"], axis=1, inplace=True)
+    df = vector_raw.set_index(["INDEX", "TYPE", "CODE_PT", "SOLN", "REF_EPOCH", "UNIT", "CONSTR", BLK_TYPE]).unstack()
+    df.reset_index(["INDEX", "UNIT", "CONSTR"], inplace=True)
+    df.attrs["aux"] = df[["INDEX", "UNIT", "CONSTR"]]
+    df.drop(["INDEX", "UNIT", "CONSTR"], axis="columns", level=0, inplace=True)
 
-    m1 = _pd.MultiIndex.from_product([df.columns, [""]])
-    m2 = _pd.MultiIndex.from_product([["VAL", "STD"], stypes])
-
-    df[list(range(len(stypes) * 2))] = _np.hstack(stypes_vals)
-
-    df.columns = m1.append(m2)
     if format == "wide":
         return snxdf2xyzdf(df, unstack=True, keep_all_soln=keep_all_soln)
     elif format == "long":
@@ -692,15 +703,15 @@ def format_snx_vector(vector_df: _pd.DataFrame) -> list:
 
     blk_bounds = _pd.Series(
         index=["APR", "EST", "NEQ"],
-        data=["SOLUTION/APRIORI\n", "SOLUTION/ESTIMATE\n", "SOLUTION/NORMAL_EQUATION_VECTOR\n"],
+        data=["SOLUTION/APRIORI", "SOLUTION/ESTIMATE", "SOLUTION/NORMAL_EQUATION_VECTOR"],
     )
 
     blk_header = _pd.Series(
         index=["APR", "EST", "NEQ"],
         data=[
-            "*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __APRIORI VALUE______ _STD_DEV___\n",
-            "*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __APRIORI VALUE______ _STD_DEV___\n",
-            "*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __RIGHT_HAND_SIDE____\n",
+            "\n*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __APRIORI VALUE______ _STD_DEV___\n",
+            "\n*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __APRIORI VALUE______ _STD_DEV___\n",
+            "\n*INDEX TYPE__ CODE PT SOLN _REF_EPOCH__ UNIT S __RIGHT_HAND_SIDE____\n",
         ],
     )
     n_blks = df.columns.levels[1].size - 1  # n blks present
@@ -729,8 +740,8 @@ def format_snx_id(snx_id):
     )
 
     return (
-        ["+SITE/ID\n"]
-        + ["*CODE PT __DOMES__ T _STATION DESCRIPTION__ APPROX_LON_ APPROX_LAT_ _APP_H_\n"]
+        ["+SITE/ID"]
+        + ["*CODE PT __DOMES__ T _STATION DESCRIPTION__ APPROX_LON_ APPROX_LAT_ _APP_H_"]
         + blk_series.to_list()
-        + ["-SITE/ID\n"]
+        + ["-SITE/ID"]
     )
